/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * Klasse zum Test der Klasse Kommazahl.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class KommazahlTest {

	/** Regel zur Auswertung erwarter Exceptions. */
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	/**
	 * Test: Attributname ermitteln.
	 */
	@Test
	public void testGetName() {
		final Kommazahl zahl = new Kommazahl("ABC", 12.0);
		Assert.assertEquals("ABC", zahl.getName());
	}

	/**
	 * Test: Konstruktor mit Attributname.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testKommazahlString() {
		Kommazahl zahl = new Kommazahl("12.0");
		Assert.assertEquals((Double) 0.0, zahl.get());
		zahl = new Kommazahl("ABC");
		Assert.assertEquals((Double) 0.0, zahl.get());

		thrown.expect(IllegalArgumentException.class);
		new Kommazahl(null);
	}

	/**
	 * Test: Konstruktor mit Attributname und Wert.
	 */
	@Test
	public void testKommazahlStringDouble() {
		final Kommazahl zahl = new Kommazahl("ABC", 12.0);
		Assert.assertEquals((Double) 12.0, zahl.get());
	}

	/**
	 * Test: Serialisierung und Deserialisierung.
	 */
	@Test
	public void testSerialization() {
		try {
			Kommazahl zahl = new Kommazahl("ABC", 12.0);
			Assert.assertEquals("ABC", zahl.getName());

			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(zahl);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			zahl = (Kommazahl) input.readObject();
			Assert.assertEquals((Double) 12.0, zahl.get());
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}
	}

	/**
	 * Test: Wert setzen.
	 */
	@Test
	public void testSet() {
		final Kommazahl zahl = new Kommazahl("ABC", 12.0);
		Assert.assertEquals((Double) 12.0, zahl.get());
		zahl.set(24.7);
		Assert.assertEquals((Double) 24.7, zahl.get());
	}

}
