/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Klasse zum Test der Klasse ParameterDatenFactory.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
@Ignore("Testdatenverteilerverbindung prüfen")
public class ParameterDatenFactoryTest {

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_FELD_1_TEXT_2 = "Feld-1-Text2";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_FELD_1_TEXT_1 = "Feld-1-Text1";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_FELD_0_TEXT_2 = "Feld-0-Text2";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_FELD_0_TEXT_1 = "Feld-0-Text1";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String ATT_NAME_KOMMAZAHL = "Kommazahl";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String ATT_NAME_GANZZAHL = "Ganzzahl";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String ATT_NAME_TEXT = "Text";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_ATTRIBUTNAME_2 = "Text2";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String ATT_NAME_FELD = "Feld";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String ATT_NAME_ATTRIBUTLISTE = "Attributliste";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_ATL_TEXT_2 = "ATL-Text2";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_ATL_TEXT_1 = "ATL-Text1";

	/**
	 * Attributname oder Wert für Testdaten.
	 */
	private static final String TEST_ATTRIBUTNAME_1 = "Text1";

	/**
	 * verwendete Datenverteilerverbindung.
	 */
	private static ClientDavInterface connection;

	// /**
	// * Systemobject zur Ausführung der Testfälle.
	// */
	// private static SystemObject object;

	/**
	 * verwendete Attributgruppe.
	 */
	private static AttributeGroup atg;

	/**
	 * verwendete Testdaten.
	 */
	private static Data daten;

	/**
	 * Datenverteilerverbindung schliessen.
	 */
	@AfterClass
	public static void closeDav() {
		if (ParameterDatenFactoryTest.connection != null) {
			ParameterDatenFactoryTest.connection.disconnect(false, "");
		}
	}

	/**
	 * Verbindung zum Dateverteiler herstellen.
	 */
	@BeforeClass
	public static void connectToDav() {

		try {
			ParameterDatenFactoryTest.connection = new ClientDavConnection(new ClientDavParameters());
			ParameterDatenFactoryTest.connection.connect();
			ParameterDatenFactoryTest.connection.login("Tester", "geheim");
		} catch (final MissingParameterException | CommunicationError | ConnectionException
				| InconsistentLoginException e) {
			Assert.fail(e.getLocalizedMessage());
		}

		// ParameterDatenFactoryTest.object =
		// ParameterDatenFactoryTest.connection
		// .getDataModel().getObject("bitctrl.parameterTest");
		ParameterDatenFactoryTest.atg = ParameterDatenFactoryTest.connection.getDataModel()
				.getAttributeGroup("atg.parametrierungsTestObjekt");

		ParameterDatenFactoryTest.daten = ParameterDatenFactoryTest.connection
				.createData(ParameterDatenFactoryTest.atg);
		ParameterDatenFactoryTest.daten.getTextValue(ParameterDatenFactoryTest.ATT_NAME_TEXT).setText("Willi");
		ParameterDatenFactoryTest.daten.getUnscaledValue(ParameterDatenFactoryTest.ATT_NAME_GANZZAHL).set(999L);
		ParameterDatenFactoryTest.daten.getUnscaledValue(ParameterDatenFactoryTest.ATT_NAME_KOMMAZAHL).set(12.234);
		ParameterDatenFactoryTest.daten.getTimeValue("Zeit").setMillis(1000000L);
		ParameterDatenFactoryTest.daten.getItem(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1)
				.setText(ParameterDatenFactoryTest.TEST_ATL_TEXT_1);
		ParameterDatenFactoryTest.daten.getItem(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2)
				.setText(ParameterDatenFactoryTest.TEST_ATL_TEXT_2);
		ParameterDatenFactoryTest.daten.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).setLength(2);
		ParameterDatenFactoryTest.daten.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(0)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1)
				.setText(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_1);
		ParameterDatenFactoryTest.daten.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(0)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2)
				.setText(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_2);
		ParameterDatenFactoryTest.daten.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(1)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1)
				.setText(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_1);
		ParameterDatenFactoryTest.daten.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(1)
				.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2)
				.setText(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_2);

	}

	/**
	 * Test: Datenverteilerdatensatz aus serialisierbarem Objekt erzeugen.
	 */
	@Test
	public void testGetData() {
		Datensatz datenSatz = (Datensatz) ParameterDatenFactory.getSerializable(ParameterDatenFactoryTest.connection,
				ParameterDatenFactoryTest.daten);

		try {
			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(datenSatz);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			datenSatz = (Datensatz) input.readObject();
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}

		final Data readData = ParameterDatenFactory.getData(ParameterDatenFactoryTest.connection, null, datenSatz);
		Assert.assertEquals("Willi", readData.getTextValue(ParameterDatenFactoryTest.ATT_NAME_TEXT).getText());
		Assert.assertEquals(999L, readData.getUnscaledValue(ParameterDatenFactoryTest.ATT_NAME_GANZZAHL).longValue());
		Assert.assertEquals(12.234,
				readData.getUnscaledValue(ParameterDatenFactoryTest.ATT_NAME_KOMMAZAHL).doubleValue(), 0);
		Assert.assertEquals(1000000L, readData.getTimeValue("Zeit").getMillis());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_ATL_TEXT_1,
				readData.getItem(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).getText());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_ATL_TEXT_2,
				readData.getItem(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).getText());

		Assert.assertEquals(2, readData.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getLength());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_1,
				readData.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(0)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).getText());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_2,
				readData.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(0)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).getText());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_1,
				readData.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(1)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).getText());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_2,
				readData.getArray(ParameterDatenFactoryTest.ATT_NAME_FELD).getItem(1)
						.getTextValue(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).getText());
	}

	/**
	 * Test: Serialisierbares Objekt erzeugen.
	 */
	@Test
	public void testGetSerializable() {

		final Datensatz datenSatz = (Datensatz) ParameterDatenFactory
				.getSerializable(ParameterDatenFactoryTest.connection, ParameterDatenFactoryTest.daten);

		Assert.assertEquals("Willi", datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_TEXT).get());
		Assert.assertEquals(999L, datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_GANZZAHL).get());
		Assert.assertEquals(12.234, datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_KOMMAZAHL).get());
		Assert.assertEquals(1000000L, datenSatz.getAttribut("Zeit").get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_ATL_TEXT_1,
				((Attributliste) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_ATL_TEXT_2,
				((Attributliste) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_ATTRIBUTLISTE))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_1,
				((Attributliste) ((Array) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_FELD)).get(0))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_0_TEXT_2,
				((Attributliste) ((Array) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_FELD)).get(0))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_1,
				((Attributliste) ((Array) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_FELD)).get(1))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_1).get());
		Assert.assertEquals(ParameterDatenFactoryTest.TEST_FELD_1_TEXT_2,
				((Attributliste) ((Array) datenSatz.getAttribut(ParameterDatenFactoryTest.ATT_NAME_FELD)).get(1))
						.getAttribut(ParameterDatenFactoryTest.TEST_ATTRIBUTNAME_2).get());
	}
}
