/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * Klasse zum Test der Klasse Array.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TestArray {

	/** Regel zur Auswertung erwarteter Exceptions. */
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	/**
	 * Attributname für Testdaten.
	 */
	private static final String ATT_NAME_ZEICHENKETTE = "Zeichenkette";

	/**
	 * Attributname für Testdaten.
	 */
	private static final String ATT_NAME_WILLI = "Willi";

	/**
	 * Attributname für Testdaten.
	 */
	private static final String ATT_NAME_DAS_FELD = "DasFeld";

	/**
	 * Attributliste für den Test.
	 */
	private List<Attribut<?>> attribute;

	/**
	 * zweite Attributliste für den Test.
	 */
	private List<Attribut<?>> attribute2;

	/**
	 * Testdaten erzeugen.
	 */
	@Before
	public void attributeAnlegen() {
		attribute = new ArrayList<>();
		attribute.add(new Kommazahl("Kommazahl", 13.657));
		attribute.add(new Ganzzahl("Ganzzahl", 678L));
		attribute.add(new Zeichenkette(TestArray.ATT_NAME_ZEICHENKETTE, "Heinz"));

		attribute2 = new ArrayList<>();
		attribute2.add(new Kommazahl("Kommazahl", 99.9));
		attribute2.add(new Ganzzahl("Ganzzahl", 999L));
		attribute2.add(new Zeichenkette(TestArray.ATT_NAME_ZEICHENKETTE, TestArray.ATT_NAME_WILLI));
	}

	/**
	 * Test: Konstruktor.
	 */
	@Test
	public void testArray() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		Assert.assertTrue(feld.size() == 0);
	}

	/**
	 * Test: Array leeren.
	 */
	@Test
	public void testClear() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		feld.add(new Zeichenkette("0", "Heinz"));
		Assert.assertTrue(feld.get().size() == 1);
		Assert.assertTrue(feld.get() instanceof List);
		Assert.assertTrue(feld.get(0) instanceof Zeichenkette);

		feld.clear();
		Assert.assertTrue(feld.get().size() == 0);
	}

	/**
	 * Test: Werte auslesen.
	 */
	@Test
	public void testGet() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		feld.add(new Attributliste("0", attribute));
		feld.add(new Attributliste("1", attribute2));
		Assert.assertTrue(feld.get().size() == 2);
		Assert.assertTrue(feld.get() instanceof List);
	}

	/**
	 * Test: Serialisierung und Deserialisierung.
	 */
	@Test
	public void testSerialization() {
		Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		feld.set(Arrays
				.asList(new Attribut<?>[] { new Attributliste("0", attribute), new Attributliste("1", attribute2) }));

		try {
			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(feld);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			feld = (Array) input.readObject();
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}

		Assert.assertEquals(TestArray.ATT_NAME_DAS_FELD, feld.getName());
		final Attributliste attListe = (Attributliste) feld.get(1);
		Assert.assertEquals(TestArray.ATT_NAME_WILLI, attListe.getAttribut(TestArray.ATT_NAME_ZEICHENKETTE).get());

	}

	/**
	 * Test: Werte setzen.
	 */
	@Test
	public void testSet() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		feld.add(new Zeichenkette("0", "Heinz"));
		Assert.assertTrue(feld.get().size() == 1);
		Assert.assertTrue(feld.get() instanceof List);
		Assert.assertTrue(feld.get(0) instanceof Zeichenkette);

		feld.set(Arrays
				.asList(new Attribut<?>[] { new Attributliste("0", attribute), new Attributliste("1", attribute2) }));
		Assert.assertTrue(feld.get().size() == 2);
		Assert.assertTrue(feld.get() instanceof List);
		Assert.assertTrue(feld.get(1) instanceof Attributliste);
	}

	/**
	 * Test: Werte mit Index setzen.
	 */
	@Test
	public void testSetIntAttributOfQextendsObject() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);

		feld.add(new Ganzzahl("Zahl", 888L));
		Assert.assertTrue(feld.get().size() == 1);
		Assert.assertEquals(888L, feld.get(0).get());

		feld.set(0, new Zeichenkette(TestArray.ATT_NAME_ZEICHENKETTE, "Heinz"));
		Assert.assertEquals("Heinz", feld.get(0).get());
	}

	/**
	 * Test: Werte mit Index setzen.
	 */
	@Test
	public void testSetIntAttributOfQextendsObjectOnInvalidIndex() {
		final Array feld = new Array(TestArray.ATT_NAME_DAS_FELD);
		thrown.expect(IndexOutOfBoundsException.class);
		feld.set(0, new Zeichenkette("0", "Heinz"));
	}

}
