/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.param;

import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.param.hierarchie.HierarchieObjekt;
import de.bsvrz.puk.param.lib.Parameter;
import de.bsvrz.puk.param.lib.ParameterInfo;

/**
 * Schnittstelle für eine Klasse zum Bereitstellen der Parameterdatensätze über
 * die Datenverteilerschnittstelle.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public interface Provider {

	/**
	 * realisiert die Anmeldung für die übergebene Parameterspezifikation, d.h.
	 * die Anmeldung meldet sich als Quelle für den Sollaspekt und als Senke für
	 * Vorgaben an.
	 *
	 * @param info
	 *            die Parameterspezifikation
	 */
	void connect(ParameterInfo info);

	/**
	 * entfernt die Anmeldung für die übergebene Parameterspezifikation, d.h.
	 * die Anmeldung als Quelle für den Sollaspekt und die Anmeldung als Senke
	 * für Vorgaben wird entfernt.
	 *
	 * @param info
	 *            die Parameterspezifikation
	 * @param entfernen
	 *            legt fest, ob Daten entfernt werden sollen oder nicht
	 */
	void disconnect(ParameterInfo info, boolean entfernen);

	/**
	 * publiziert die Parameter für alle Objekte, deren Daten durch die Daten
	 * des übergebenen Hierarchieobjekt beeinflusst werden neu.
	 *
	 * @param hierarchieObjekt
	 *            das Hierarchieobjekt
	 * @param parameter
	 *            die zu publizierenden Parameter
	 */
	void publiziereHierarchieParameter(HierarchieObjekt hierarchieObjekt, Parameter parameter);

	/**
	 * publiziert einen lokalen Parameter.
	 *
	 * @param parameter
	 *            die Parameterdaten
	 */
	void publiziereLokalenParameter(Parameter parameter);

	/**
	 * publiziert die Parameterdaten für alle Objekte, des übergebenen Typs neu.
	 *
	 * @param typ
	 *            der Typ der Objekte
	 * @param parameter
	 *            die Parameterdaten
	 */
	void publiziereTypParameter(SystemObjectType typ, Parameter parameter);

	/**
	 * ermittelt, ob die angegebene Parameterspezifikation versorgt wird.
	 *
	 * @param info
	 *            die Spezifikation
	 * @return der Status
	 */
	boolean isProvided(ParameterInfo info);
}
