/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.param;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.param.simulation.SimulationsVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Repräsentation des Software-Moduls "Verwaltung".<br>
 * Das Modul startet das eigentliche Parametrierungsmodul und das
 * Verwaltungsmodul für die Bereitstellung von Parametern für Simulationen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
class Verwaltung {

	/** der Logger für Debug-Ausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	/** das verwendete Parametrierungsmodul. */
	private final ParametrierungsModul parametrierung;

	/** das Modul zur Verwaltung der Simulationen. */
	private final SimulationsVerwaltung simulationsVerwaltung;

	/**
	 * der Konstruktor erzeugt das Vermaltungsmodul.
	 *
	 * @param param
	 *            die verwendete Parametrierungsapplikation
	 * @param persistenz
	 *            das verwendete Persistenzmodul
	 * @param parametrierungsObjekt
	 *            das Systemobjekt, das die Parametrierungsapplikation innerhalb
	 *            der Datenverteilerkonfiguration repräsentiert
	 * @param oldDefault
	 *            bestimmt, ob die Definition von Standardparametern für
	 *            Konfigurationsobjekte mit dem Aspekt "asp.parameterDefault"
	 *            unterstützt wird
	 */
	Verwaltung(final ParamApp param, final Persistenz persistenz, final SystemObject parametrierungsObjekt,
			final boolean oldDefault) {

		Verwaltung.LOGGER.info(
				"Starte Parameterverwaltung für Parametrierungsobjekt: " + parametrierungsObjekt.getNameOrPidOrId());
		parametrierung = new Parametrierung(param, parametrierungsObjekt, persistenz, oldDefault);
		simulationsVerwaltung = new SimulationsVerwaltung(parametrierung);
		Verwaltung.LOGGER.config("Parametrierungsmodul und Simulationsverwaltung gestartet");
	}

	/**
	 * beendet das Verwaltungsmodul.
	 */
	void close() {
		if (parametrierung != null) {
			parametrierung.close();
		}
	}

	/**
	 * liefert das angelegte Modul zur Simulationsverwaltung.
	 *
	 * @return das Modul
	 */
	SimulationsVerwaltung getSimulationsVerwaltung() {
		return simulationsVerwaltung;
	}
}
